/*
 * This file contains D-Bus client proxy classes generated by qt4-client-gen.py.
 *
 * This file can be distributed under the same terms as the specification from
 * which it was generated.
 */

#ifndef IN_TELEPATHY_QT4_HEADER
#error IN_TELEPATHY_QT4_HEADER
#endif

#include <TelepathyQt4/Types>

#include <QtGlobal>

#include <QString>
#include <QObject>
#include <QVariant>

#include <QDBusPendingReply>

#include <TelepathyQt4/Client/AbstractInterface>
#include <TelepathyQt4/Client/DBusProxy>

// basically the same as GLib's G_GNUC_DEPRECATED
#ifndef TELEPATHY_GNUC_DEPRECATED
#   if defined(Q_CC_GNUC) && __GNUC__ >= 4
#       define TELEPATHY_GNUC_DEPRECATED __attribute__((__deprecated__))
#   else
#       define TELEPATHY_GNUC_DEPRECATED
#   endif
#endif
namespace Telepathy
{
namespace Client
{

/**
 * \class MediaStreamHandlerInterface
 * \headerfile TelepathyQt4/Client/media-stream-handler.h <TelepathyQt4/Client/MediaStreamHandler>
 * \ingroup clientmstrh
 *
 * Proxy class providing a 1:1 mapping of the D-Bus interface "org.freedesktop.Telepathy.Media.StreamHandler."
 */
class MediaStreamHandlerInterface : public Telepathy::Client::AbstractInterface
{
    Q_OBJECT

public:
    /**
     * Returns the name of the interface "org.freedesktop.Telepathy.Media.StreamHandler", which this class
     * represents.
     *
     * \return The D-Bus interface name.
     */
    static inline const char *staticInterfaceName()
    {
        return "org.freedesktop.Telepathy.Media.StreamHandler";
    }

    /**
     * Creates a MediaStreamHandlerInterface associated with the given object on the session bus.
     *
     * \param serviceName Name of the service the object is on.
     * \param objectPath Path to the object on the service.
     * \param parent Passed to the parent class constructor.
     */
    MediaStreamHandlerInterface(
        const QString& serviceName,
        const QString& objectPath,
        QObject* parent = 0
    );

    /**
     * Creates a MediaStreamHandlerInterface associated with the given object on the given bus.
     *
     * \param connection The bus via which the object can be reached.
     * \param serviceName Name of the service the object is on.
     * \param objectPath Path to the object on the service.
     * \param parent Passed to the parent class constructor.
     */
    MediaStreamHandlerInterface(
        const QDBusConnection& connection,
        const QString& serviceName,
        const QString& objectPath,
        QObject* parent = 0
    );

    /**
     * Creates a MediaStreamHandlerInterface associated with the same object as the given proxy.
     *
     * \param proxy The proxy to use. It will also be the QObject::parent()
     *               for this object.
     */
    MediaStreamHandlerInterface(Telepathy::Client::DBusProxy *proxy);

public Q_SLOTS:
    /**
     * Begins a call to the D-Bus method "CodecChoice" on the remote object.
     * 
     * Inform the connection manager of codec used to receive data.
     */
    inline QDBusPendingReply<> CodecChoice(uint codecID)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(codecID);
        return asyncCallWithArgumentList(QLatin1String("CodecChoice"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "Error" on the remote object.
     * 
     * Inform the connection manager that an error occured in this stream. The 
     * connection manager should emit the StreamError signal for the stream on 
     * the relevant channel, and remove the stream from the session.
     *
     * \param errorCode
     *     
     *     ID of error, from the MediaStreamError enumeration
     *
     * \param message
     *     
     *     String describing the error
     */
    inline QDBusPendingReply<> Error(uint errorCode, const QString& message)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(errorCode) << QVariant::fromValue(message);
        return asyncCallWithArgumentList(QLatin1String("Error"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "NativeCandidatesPrepared" on the remote object.
     * 
     * Informs the connection manager that all possible native candisates have 
     * been discovered for the moment.
     */
    inline QDBusPendingReply<> NativeCandidatesPrepared()
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        return asyncCall(QLatin1String("NativeCandidatesPrepared"));
    }

    /**
     * Begins a call to the D-Bus method "NewActiveCandidatePair" on the remote object.
     * 
     * Informs the connection manager that a valid candidate pair has been 
     * discovered and streaming is in progress.
     */
    inline QDBusPendingReply<> NewActiveCandidatePair(const QString& nativeCandidateID, const QString& remoteCandidateID)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(nativeCandidateID) << QVariant::fromValue(remoteCandidateID);
        return asyncCallWithArgumentList(QLatin1String("NewActiveCandidatePair"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "NewNativeCandidate" on the remote object.
     * 
     * Inform this MediaStreamHandler that a new native transport candidate 
     * has been ascertained.
     *
     * \param candidateID
     *     
     *     String identifier for this candidate
     *
     * \param transports
     *     
     *     Array of transports for this candidate, with fields:
     *     <ul>
     *       <li>component number</li>
     *       <li>IP address (as a string)</li>
     *       <li>port</li>
     *       <li>base network protocol (one of the values of MediaStreamBaseProto)</li>
     *       <li>proto subtype (e.g. RTP)</li>
     *       <li>proto profile (e.g. AVP)</li>
     *       <li>our preference value of this transport (double in range 0.0-1.0
     *         inclusive); 1 signals the most preferred transport</li>
     *       <li>transport type, one of the values of MediaStreamTransportType</li>
     *       <li>username if authentication is required</li>
     *       <li>password if authentication is required</li>
     *     </ul>
     */
    inline QDBusPendingReply<> NewNativeCandidate(const QString& candidateID, const Telepathy::MediaStreamHandlerTransportList& transports)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(candidateID) << QVariant::fromValue(transports);
        return asyncCallWithArgumentList(QLatin1String("NewNativeCandidate"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "Ready" on the remote object.
     * 
     * Inform the connection manager that a client is ready to handle this 
     * StreamHandler. Also provide it with info about all supported codecs.
     *
     * \param codecs
     *     
     *     Locally-supported codecs.
     */
    inline QDBusPendingReply<> Ready(const Telepathy::MediaStreamHandlerCodecList& codecs)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(codecs);
        return asyncCallWithArgumentList(QLatin1String("Ready"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "SetLocalCodecs" on the remote object.
     * 
     * <p>Used to provide codecs after Ready(), so the media client can go
     *   ready for an incoming call and exchange candidates/codecs before
     *   knowing what local codecs are available.</p>
     * 
     * <p>This is useful for gatewaying calls between two connection managers.
     *   Given an incoming call, you need to call
     *   <tp:member-ref>Ready</tp:member-ref> to get the remote codecs before
     *   you can use them as the &quot;local&quot; codecs to place the outgoing call,
     *   and hence receive the outgoing call's remote codecs to use as the
     *   incoming call's &quot;local&quot; codecs.</p>
     * 
     * <p>In this situation, you would pass an empty list of codecs to the
     *   incoming call's Ready method, then later call SetLocalCodecs on the
     *   incoming call in order to respond to the offer.</p>
     *
     * \param codecs
     *     
     *     Locally-supported codecs
     */
    inline QDBusPendingReply<> SetLocalCodecs(const Telepathy::MediaStreamHandlerCodecList& codecs)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(codecs);
        return asyncCallWithArgumentList(QLatin1String("SetLocalCodecs"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "StreamState" on the remote object.
     * 
     * Informs the connection manager of the stream&apos;s current state, as 
     * as specified in Channel.Type.StreamedMedia::ListStreams.
     */
    inline QDBusPendingReply<> StreamState(uint state)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(state);
        return asyncCallWithArgumentList(QLatin1String("StreamState"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "SupportedCodecs" on the remote object.
     * 
     * Inform the connection manager of the supported codecs for this session. 
     * This is called after the connection manager has emitted SetRemoteCodecs 
     * to notify what codecs are supported by the peer, and will thus be an 
     * intersection of all locally supported codecs (passed to Ready) and 
     * those supported by the peer.
     *
     * \param codecs
     *     
     *     Locally supported codecs.
     */
    inline QDBusPendingReply<> SupportedCodecs(const Telepathy::MediaStreamHandlerCodecList& codecs)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(codecs);
        return asyncCallWithArgumentList(QLatin1String("SupportedCodecs"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "CodecsUpdated" on the remote object.
     * 
     * Inform the connection manager that the parameters of the supported 
     * codecs for this session have changed. The connection manager should 
     * send the new parameters to the remote contact. This is required for 
     * H.264 and Theora, for example.
     *
     * \param codecs
     *     
     *     Locally supported codecs, which SHOULD be the same as were 
     *     previously in effect, but possibly with different parameters.
     */
    inline QDBusPendingReply<> CodecsUpdated(const Telepathy::MediaStreamHandlerCodecList& codecs)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(codecs);
        return asyncCallWithArgumentList(QLatin1String("CodecsUpdated"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "HoldState" on the remote object.
     * 
     * Notify the connection manager that the stream&apos;s hold state has 
     * been changed successfully in response to SetStreamHeld.
     *
     * \param held
     *     
     *     If true, the stream is now on hold.
     */
    inline QDBusPendingReply<> HoldState(bool held)
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(held);
        return asyncCallWithArgumentList(QLatin1String("HoldState"), argumentList);
    }

    /**
     * Begins a call to the D-Bus method "UnholdFailure" on the remote object.
     * 
     * Notify the connection manager that an attempt to reacquire the 
     * necessary hardware or software resources to unhold the stream, in 
     * response to SetStreamHeld, has failed.
     */
    inline QDBusPendingReply<> UnholdFailure()
    {
        if (!invalidationReason().isEmpty()) {
            return QDBusPendingReply<>(QDBusMessage::createError(
                invalidationReason(),
                invalidationMessage()
            ));
        }


        return asyncCall(QLatin1String("UnholdFailure"));
    }

Q_SIGNALS:
    /**
     * Represents the signal "AddRemoteCandidate" on the remote object.
     * 
     * Signal emitted when the connection manager wishes to inform the client 
     * of a new remote candidate.
     *
     * \param candidateID
     *     
     *     String identifier for this candidate
     *
     * \param transports
     *     
     *     Array of transports for this candidate with fields, as defined in 
     *     NewNativeCandidate
     */
    void AddRemoteCandidate(const QString& candidateID, const Telepathy::MediaStreamHandlerTransportList& transports);

    /**
     * Represents the signal "Close" on the remote object.
     * 
     * Signal emitted when the connection manager wishes the stream to be 
     * closed.
     */
    void Close();

    /**
     * Represents the signal "RemoveRemoteCandidate" on the remote object.
     * 
     * Signal emitted when the connection manager wishes to inform the client 
     * that the remote end has removed a previously usable candidate. It 
     * seemed like a good idea at the time, but wasn&apos;t.
     *
     * \param candidateID
     *     
     *     String identifier for remote candidate to drop
     */
    void RemoveRemoteCandidate(const QString& candidateID);

    /**
     * Represents the signal "SetActiveCandidatePair" on the remote object.
     * 
     * Emitted by the connection manager to inform the client that a valid 
     * candidate pair has been discovered by the remote end and streaming is 
     * in progress.
     */
    void SetActiveCandidatePair(const QString& nativeCandidateID, const QString& remoteCandidateID);

    /**
     * Represents the signal "SetRemoteCandidateList" on the remote object.
     * 
     * Signal emitted when the connection manager wishes to inform the client 
     * of all the available remote candidates at once.
     *
     * \param remoteCandidates
     *     
     *     A list of candidate id and a list of transports as defined in 
     *     NewNativeCandidate
     */
    void SetRemoteCandidateList(const Telepathy::MediaStreamHandlerCandidateList& remoteCandidates);

    /**
     * Represents the signal "SetRemoteCodecs" on the remote object.
     * 
     * Signal emitted when the connection manager wishes to inform the client 
     * of the codecs supported by the remote end. 	If these codecs are 
     * compatible with the remote codecs, then the client must call 
     * SupportedCodecs, otherwise call Error.
     *
     * \param codecs
     *     
     *     Codecs supported by the remote peer.
     */
    void SetRemoteCodecs(const Telepathy::MediaStreamHandlerCodecList& codecs);

    /**
     * Represents the signal "SetStreamPlaying" on the remote object.
     * 
     * If emitted with argument TRUE, this means that the connection manager 
     * wishes to set the stream playing; this means that the streaming 
     * implementation should expect to receive data. If emitted with argument 
     * FALSE this signal is basically meaningless and should be ignored. 
     * We&apos;re very sorry.
     */
    void SetStreamPlaying(bool playing);

    /**
     * Represents the signal "SetStreamSending" on the remote object.
     * 
     * Signal emitted when the connection manager wishes to set whether or not 
     * the stream sends to the remote end.
     */
    void SetStreamSending(bool sending);

    /**
     * Represents the signal "StartTelephonyEvent" on the remote object.
     * 
     * Request that a telephony event (as defined by RFC 4733) is transmitted 
     * over this stream until StopTelephonyEvent is called.
     *
     * \param event
     *     
     *     A telephony event code as defined by RFC 4733.
     */
    void StartTelephonyEvent(uchar event);

    /**
     * Represents the signal "StopTelephonyEvent" on the remote object.
     * 
     * Request that any ongoing telephony events (as defined by RFC 4733) 
     * being transmitted over this stream are stopped.
     */
    void StopTelephonyEvent();

    /**
     * Represents the signal "SetStreamHeld" on the remote object.
     * 
     * <p>Emitted when the connection manager wishes to place the stream on
     *   hold (so the streaming client should free hardware or software
     *   resources) or take the stream off hold (so the streaming client
     *   should reacquire the necessary resources).</p>
     * 
     * <p>When placing a channel's streams on hold, the connection manager
     *   SHOULD notify the remote contact that this will be done (if
     *   appropriate in the protocol) before it emits this signal.</p>
     * 
     * <tp:rationale>
     *   <p>It is assumed that relinquishing a resource will not fail.
     *     If it does, the call is probably doomed anyway.</p>
     * </tp:rationale>
     * 
     * <p>When unholding a channel's streams, the connection manager
     *   SHOULD emit this signal and wait for success to be indicated
     *   via HoldState before it notifies the remote contact that the
     *   channel has been taken off hold.</p>
     * 
     * <tp:rationale>
     *   <p>This means that if a resource is unavailable, the remote
     *     contact will never even be told that we tried to acquire it.</p>
     * </tp:rationale>
     *
     * \param held
     *     
     *     If true, the stream is to be placed on hold.
     */
    void SetStreamHeld(bool held);

protected:
    virtual void invalidate(Telepathy::Client::DBusProxy *, const QString &, const QString &);
};
}
}
